package com.jpattern.orm.generator.reflection;

import java.lang.reflect.Field;
import java.lang.reflect.Method;

import com.jpattern.orm.classmapper.IColumn;
import com.jpattern.orm.generator.ResultSetMethodHelper;
import com.jpattern.orm.generator.wrapper.TypeWrapper;

/**
 * 
 * @author Francesco Cina'
 *
 * Apr 1, 2012
 */
public class AReflectionManipulatorFactory {

	public static AGetFieldManipulator getGetFieldManipulator(IColumn column) throws SecurityException, NoSuchMethodException {
		if ( column.getGetter() == null) {
			return getFieldGetFieldManipulator(column);
		}
		return getGetterGetFieldManipulator(column);
	}

	private static AGetFieldManipulator getFieldGetFieldManipulator(IColumn column) throws SecurityException, NoSuchMethodException {
		final TypeWrapper<Object, Object> wrapper = getWrapper(column);
		if ( wrapper != null ) {
			return new FieldWrappedGetFieldManipulator(column.getField(), wrapper);
		}
		return new FieldGetFieldManipulator(column.getField());
	}

	private static AGetFieldManipulator getGetterGetFieldManipulator(IColumn column) throws SecurityException, NoSuchMethodException {
		final TypeWrapper<Object, Object> wrapper = getWrapper(column);
		if ( wrapper != null ) {
			return new GetterWrappedGetFieldManipulator(column.getGetter(), wrapper);
		}
		return new GetterGetFieldManipulator(column.getGetter());
	}


	public static ASetFieldManipulator getSetFieldManipulator(IColumn column) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, NoSuchFieldException {
		final Field field = column.getField();
		final Method resultSetGetterMethodString = ResultSetMethodHelper.findResultSetGetterForString(field.getType());
		final Method resultSetGetterMethodInt = ResultSetMethodHelper.findResultSetGetterForInt(field.getType());
		if ( column.getSetter() == null) {
			return getFieldSetFieldManipulator(column, resultSetGetterMethodString, resultSetGetterMethodInt);
		}
		return getSetterSetFieldManipulator(column, resultSetGetterMethodString, resultSetGetterMethodInt);
	}

	private static ASetFieldManipulator getFieldSetFieldManipulator(IColumn column, Method resultSetGetterMethodString, Method resultSetGetterMethodInt) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, NoSuchFieldException {
		final TypeWrapper<Object, Object> wrapper = getWrapper(column);
		if ( wrapper != null ) {
			return new FieldWrappedSetFieldManipulator(column.getField(), resultSetGetterMethodString, resultSetGetterMethodInt, wrapper);
		}
		return new FieldSetFieldManipulator(column.getField(), resultSetGetterMethodString, resultSetGetterMethodInt);
	}

	private static ASetFieldManipulator getSetterSetFieldManipulator(IColumn column, Method resultSetGetterMethodString, Method resultSetGetterMethodInt) throws SecurityException, NoSuchMethodException {
		final TypeWrapper<Object, Object> wrapper = getWrapper(column);
		if ( wrapper != null ) {
			return new SetterWrappedSetFieldManipulator(column.getSetter(), resultSetGetterMethodString, resultSetGetterMethodInt, wrapper);
		}
		return new SetterSetFieldManipulator(column.getSetter(), resultSetGetterMethodString, resultSetGetterMethodInt);
	}

	public static VersionManipulator getVersionManipulator(IColumn column, FieldManipulator fieldManipulator) throws SecurityException, NoSuchMethodException {
		final Class<?> rsType = ResultSetMethodHelper.findResultSetGetterForString(column.getField().getType()).getReturnType();
		return new ReflectionVersionManipulator(rsType, fieldManipulator);
	}

	public static GeneratorManipulator getGeneratorManipulator(IColumn column, FieldManipulator fieldManipulator, long[] validValues) throws SecurityException, NoSuchMethodException {
		final Class<?> rsType = ResultSetMethodHelper.findResultSetGetterForString(column.getField().getType()).getReturnType();
		return new ReflectionGeneratorManipulator(rsType, fieldManipulator, validValues);
	}

	@SuppressWarnings("unchecked")
	private static TypeWrapper<Object, Object> getWrapper(IColumn column) throws SecurityException, NoSuchMethodException {
		if ( ResultSetMethodHelper.isWrappedType(column.getField().getType()) ) {
			return (TypeWrapper<Object, Object>) ResultSetMethodHelper.getWrapper(column.getField().getType());
		}
		return null;
	}
}
