package com.jpattern.orm.generator.reflection;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import com.jpattern.orm.exception.OrmConfigurationException;

/**
 * 
 * Get the value of a field directly accessing his value
 * 
 * @author Francesco Cina'
 *
 * Mar 31, 2012
 */
public class FieldSetFieldManipulator extends ASetFieldManipulator {

	private final Field field;

	public FieldSetFieldManipulator(Field field, Method resultSetGetterMethodString, Method resultSetGetterMethodInt) {
		super(resultSetGetterMethodString, resultSetGetterMethodInt);
		this.field = field;
		field.setAccessible(true);

		if (Modifier.isFinal(field.getModifiers()) ) {
			throw new OrmConfigurationException("Field [" + field.getName() + "] of class [" + field.getDeclaringClass() + "] is marked FINAL. His value cannot be managed by JPOrm. Please remove the 'final' modifier.");
		}

	}

	@Override
	public void setValue(Object bean, Object value) throws IllegalArgumentException, IllegalAccessException {
		this.field.set(bean, value);
	}

}
